###################################################################
###################################################################          
##                                                               ## 
##                       Crysfire_2020                           ##
##                                                               ##
##                  Ron Ghosh,     April  2024                   ##
##                                                               ##
###################################################################
#
#
# 
# Version 1.0.8 - now has versioning after edits
# Version 1.0.7 - fonts now scalable; small errors in new directory setting
#                 programs should close files better at end of run
# Version 1.0.6 - small corrections better error treatment  August 2022
# Version 1.0.5 - false start .CRroot set to default. Browser for html file
#                 July 20
# Version 1.0.4 - task management added to kill off recalcitrant tasks
#                 psta reintroduced for Windows to create separate process
#                 otherwise tcl-subtask consumes all cpu-time
#                 separates new file/edit file
#                 introduces table input option
# Version 1.0.3 - relative paths permitted in starting command
#                  (more useful for OSX etc - Windows has full names 
#		   in icon shortcut)
# Version 1.0.2 - first version on CCP!4
# 
#
# Fortran Uses cr2020.dfp for indexing program default paras and matching index progs
#      cr2020.dfx file transfers optional settings (wavelength, zero offset)
# Crysfire uses crdfp and crdfx as working files - copied to the above when a
#	new version ver is created; project_ver.DFX and project_ver.DFP are made as
#	future reference copies at this time
# program in use with ID values  ttdd ttssdd tzero delt sect fils newdir savdir
#	llid lid lexe lnam  lists of IDs and exenames 
#	pcur 	current running prog 
#	bproj 	baseproject directory name and project name
#	crversion current data version
#	pdir work/bproj dir   fproj project files 
#
# 	ROUTINES
#	
#	chknew	checks for exising project
#       chkres  check resume project is ok
#	chkbut	update active output buttons (output files in project)
#	chkppp	checks list for good peaks - copies/ignores * lines
#	chktht	checks new.PPP table for valid input
#       chkwrt  checks file is writeable
#	debug	output on entry to routines
#	disbut	disbales all view buttons
#	edidef  edit default parameter file cr2020.dfp
#	edtpar	edit title/zero etc cr2020.dfx
#	edtpro  edit project peak file proj.PPP etc
#	edttht	checks edit window and saves
# 	endtsk  ends subtask after interruption 
#       ftini   sets up font initially
#       ftset   rescales/rests fonts
#       ftupt   actually modifies fonts
#	ini	initialise routine , set cwd
#	logit	logs mesaage
#	maksum  concat summaries, sort, update vratio, run clepag14
#	message	ddt window
#	newcrrt set new work directory
#	newpron	gets newprojectname  
#	newprof	creates new project file new.PPP
#	npslcl	selects column from input table
#	npsbak	retores original table input
#       newmain creates new working root dir
#	resproj resumeexisting project
#	runpgm	runs specific indexing program
#	runpxe  runs program and shows clock, with interrupt button
#	savcf	save configuration file
#	setupallp setup program buttons
#       shofil	sho list of files in project
#	shopro	sho list of projects
# 	shosums sho Lepage summaries
#       texinfo    writes texinfo bar    
#	uphead	update header in main panel
#	viewfile  starts generic file viewer
#	vuout	show full output of this run
#	vusum	show summary of this run
#	vusums  show global summary
#		versions(v1.0.8)
#	vget 	get current version and edit status
#	vput	update v and edit
#	vfil	create version files
#	vnew	decide on new version
#
#
#
#      GLOBALS
#
global ddt cfont homedir pform sdir proj pdir proroot header resbut ddt
global cfdir conffile vufile atend  vdate work zoff wav titl filerr newp
global lbutt lext sheader sheader1 sheader0 savroot fwrite ddt pross resok
global dtheta wmol newroot vversion
global pexenam pgexe killd psta newttl lthet
global ttl filerr emptyttl defcol intab seldon setit browsr min2t max2t
global debunit fnames fsizes setscal
global crversion vrw bproj edtd nall nuse ltxt badval badrge badseq

###################### debug
proc debug { txt } {
global ddt cfont homedir pform sdir proj pdir proroot header resbut ddt
global cfdir conffile vufile atend  vdate work zoff wav titl filerr newp
global lbutt lext sheader sheader1 sheader0 savroot fwrite ddt pross resok
global dtheta wmol newroot vversion ttl filerr emtyttl defcol fproj
global debunit
# ddt is set at start of main...   crdebug is placed in HOME directory
# insert gloabl to show in each called routine
if { $ddt == 1 } { puts $debunit ">>> $txt +++ "  }
}


###################### chkbut  check presence of output files to set up view buttons
proc chkbut { } {
global lbutt lext proj ddt
# simplest to check all files...rather than last couple changed !!
debug "chkbut    "
foreach b $lbutt {
$b configure -state disabled
}
.all.sumo configure -state disabled
.all.summ configure -state disabled
set nn [ llength $lext ]
set ii -1
foreach extn $lext {
incr ii
set fnm "$proj.$extn"
if { [ file exists $fnm ] } {
set butt [ lindex $lbutt $ii ]
$butt configure -state normal
#debug "chkbut $ii  $fnm found button $butt set " 
}
}
if { [file exists $proj.CLO ] } {
.all.sumo configure -state normal
}
if { [file exists $proj.CLM ] } {
.all.summ configure -state normal
}

} 

########################### chknew  checks for exising project
proc chknew { } {
global pproj fproj work homedir cfdir newp okres titl wav zoff resbut ddt
global dtheta wmol bproj
debug "chknew   "
#.main.tt configure -text "Checking for files "
texinfo "blue" "Checking for files "
# workdir is defined usually
debug  "chknew  proj is $newp" 
set okres 0
set newpdir "$work/$newp"
debug "chknew  newpdir is $work/$newp"
set mm ""
if { [ file isdirectory $newpdir ] == 0 } {set mm "No project $newpdir found" }
if { $mm == "" } {
if { [ file exists $newpdir/$newp.PPP ] == 0 } { set mm "No peak file $newpdir/$newp.PPP"}
}
if { $mm == "" } {
if { [ file exists $newpdir/cr2020.dfp ] == 0 } { set mm "Project file $work$newpdir damaged"} 
}
if { $mm != "" } {
#.main.tt configure -text $mm
texinfo "red" $mm
return
}
if { $mm == "" } {
cd $newpdir
set bproj $newp
debug "chknew bproj and newp $bproj $newp "}
#
#reread dpx file
set ff [ open crdfx r ]
gets $ff titl
gets $ff zoff
gets $ff dtheta
gets $ff wav
close $ff
set fproj $bproj.PPP
set okres 1
set pdef $work/.CRproj
set ff [ open $pdef w ]
puts $ff $bproj
close $ff
debug "chknew setting crproj file $bproj  newp $newp" 
uphead 0
#.main.tt configure -text " ""$newp"" set to be current project "
texinfo "blue" "Current project set to $bproj "
chkbut
}


###################### chkres  checks resume project is ok
proc chkres { newp } {
global work titl wav zoff ddt resok proj 
global dtheta wmol bproj crversion edtd hedited
# checks for config files and data in new file ans updates header
# error tetx into .rpj.e label; should still be there
debug "chkres  newp is $newp work $work" 
.rpj.e configure -text "" 
debug "chkres  cd to $work/$newp" 
if { [file isdirectory $work/$newp ] != 1 } {
.rpj.e configure -text "$newp is not a valid  directory " -fg "red" 
return
}
set resok 0
set dfx "dfx_bad"
set dfp "dfp_bad"
set PPP "PPP__bad"
if { [file exists $work/$newp/pversion] != 1 }  {
# perhaps prior to v108 ned to make crdfx, crdfp
catch {
file copy $work/$newp/cr2020.dfx $work/$newp/crdfx
file copy $work/$newp/cr2020.dfp $work/$newp/crdfp
} }
if { [ file exists $work/$newp/crdfx ] ==1 } {
set dfx "dfx_ok"
} else {
incr resok
}  
if { [ file exists $work/$newp/crdfp ] ==1 } {
set dfp "dfp_ok"
} else {
incr resok
} 
if { [file exists $work/$newp/$newp.PPP ] == 1 }  {
set PPP "PPP_ok"
} else {
incr resok
}
if { $resok > 0 } {
.rpj.e configure -text "resume $newp errors $PPP $dfx $dfp" -fg "red" 
incr resok
return
}
debug "chkres updating headers etc"
set titl "No title found"
set zoff 0.0
set dtheta 0.03
set wav 1.54036
catch {
set ff [ open $work/$newp/crdfx r ]
gets $ff titl
gets $ff zoff
gets $ff dtheta
gets $ff wav 
close $ff }
set ff [ open $work/.CRproj w ]
puts $ff $newp
close $ff
cd $work/$newp
set bproj $newp
# reset version or set to 1
vget
set proj $bproj\_$crversion
texinfo "blue" "Current project set to $bproj "
# no edits necessary ?
set edtd 0
set hedited ""
uphead 0
chkbut
update
debug "chkres now in $work/$newp bproj $bproj"
}

###################### chkwrt  checks dir is writeable
proc chkwrt { fnam } {
global fwrite ddt
debug "chkwrt"
set fwrite 0
set fl $fnam/.test
set test ""
catch {
set ff [ open $fl w ]
puts $ff $test
close $ff
file delete $fl
set test "done"
debug "Chkwrt $fnam $test " 
}
if { $test == "done" } { set fwrite 1 }
}

################       chktht     checks &  save data to project peak file

proc chktht { } {
# some redundancy with etdtht this is called from newprof
global proj filerr fproj titl wav zoff dtheta cfont ddt  ttl lthet  lthet newttl min2t max2t
# first check values are reals 2-100 degrees return in list lthet
global bproj ltxt
debug "chktht last filerr $filerr" 
set res [ .nwp.f.t.t get 2.0 end ]
set lreal ""
set ltxt [ split $res "\n" ]
set lthet ""
set prev  0.
set badrge 0
set badval 0
set badseq 0
set real 0.
set sumerr 0
set filerr ""
set errtxt ""
.nwp.f.lerr configure -text $errtxt
chkppp
if { $filerr != ""  } {
set errtxt "Error line#(0=OK) value/range/sequence $badval/$badrge/$badseq"
.nwp.f.lerr configure -text $errtxt -fg red
return }
destroy .nwp
# releases grab on window and lets treatment proceed
}
###################### chkppp  checks peak is in seq/range - skips if *
proc chkppp { } {
global filerr ltxt lthet min2t max2t badval badrge badseq
debug "chkppp"
#set lentxt [ llength $ltxt ]
#incr lentxt -2
set lthet ""
set filerr ""
set badval 0
set badseq 0
set badrge 0
set prev  0.
set i 0
foreach ln $ltxt {
#clear unwanted returns at end of list
incr i
if { $ln != "" } {
set star "*"
# filter data later
# look for asterisk
if { [ string first * $ln ] < 0 } { set star "" }
if { $star == "" } {
# check for range
set stat [ scan $ln "%f" real ]
# format selects real field
if { $stat != 1 } { set badval $i }
if { $real < $min2t | $real > $max2t } { set badrge $i }
# range from synchrotron to neutron difractometer
if { $real <= $prev } { set badseq $i }
set sumerr [ expr $badval+$badseq+$badrge]
#  ok to write out file 
if { $sumerr != 0 } { set filerr "$badval/$badseq/$badrge" }
if { $sumerr > 0 } break
lappend lthet $real
set prev $real
} else {
# if * then just copy
lappend lthet $ln
} } }
# so we have lthet with checked reals and commented out unedited lines 
}


###################### disbut  disables all result file view buttons
proc disbut { } {
global lbutt
debug "disbut "
foreach b $lbutt {
$b configure -state disabled
}
}

###################### edidef  edit default parameter file CR2020.DFP
proc edidef { } {
global  cfont atend sheader sheader1 sdir ddt
global edtd hedited logtxt 
debug "edidef  "
toplevel .et
grab set  .et
focus .et
frame .et.lbb
set vufil "crdfp"
# not global here
set atend 1
text .et.lbb.tb -width 80 -height 25 -font $cfont -xscrollcommand \
".et.lbb.scrollx set" -yscrollcommand ".et.lbb.scrolly set"  -wrap none -font $cfont

#listbox .et.lbb.lb -width 80 -height 25 -font $cfont -yscrollcommand ".et.lbb.scrolly set " \
#-xscrollcommand ".et.lbb.scrollx set " -background white
scrollbar .et.lbb.scrolly -command " .et.lbb.tb yview " -orient v
scrollbar .et.lbb.scrollx -command " .et.lbb.tb xview " -orient h
wm title .et $vufil
set found  [ file exists $vufil ]
set ppwd [ pwd ]
debug "edidef vufil $ppwd $vufil $found" 
if { [catch { open $vufil r } inf ] == 0  } {
if { $sheader != "" } {
.et.lbb.tb insert end "$sheader1\n" }
while { [gets $inf line ] >= 0 } {
.et.lbb.tb insert end "$line\n"
#puts $line
}
close $inf
#  view logfile from end
if { $atend == 0 } {.et.lbb.tb yview end }
}
frame .et.bbb
button .et.bbb.but -text "Dismiss" -padx 5 -command { destroy .et}
button .et.bbb.br -text "Replace file" -padx 5 -command {
set res [ .et.lbb.tb get 1.0 end ]
set ltxt [ split $res "\n" ]
set lentxt [ llength $ltxt ]
set out [ open crdfp w ]
foreach lin $ltxt {
puts $out $lin
}
close $out
set edtd 1
set hedited " edited"
vput
set logtxt "Parameter file crdfp modified "
logit
#.main.tt configure -text "Project default data file cr2020.dfp rewritten"
texinfo "blue" "Project default data file cr2020.dfp rewritten"
destroy .et
}
button .et.bbb.restore -text "Reread master copy" -padx 5 -command {
set infl $sdir/cr2020.dfp
set inf [open $infl r ]
set outf [ open cvdfp w ]
while { [gets $inf lin ] >= 0 } { puts $outf $lin }
close $inf
close $outf
#.main.tt configure -text "Default data file cr2020.dfp reestored"
.texinfo "blue" "Default data file cr2020.dfp reestored"
destroy .et
}
pack .et.lbb.scrollx -side bottom -fill x
pack .et.lbb.tb -side left
pack .et.lbb.scrolly -side right -fill y
pack .et.bbb.br .et.bbb.restore .et.bbb.but -side left -expand 1 
pack .et.lbb
pack .et.bbb
tkwait window .et
}


###################### edtpar  edit title/wave/zero-off/dtheta
proc edtpar { } {
global titl wav zoff bproj ddt
global dtheta wmol
global edtd
debug "edtpar  "
toplevel .edp
# message box
label .edp.l -text ""
frame .edp.f
label .edp.f.t -width 20 -text "Title"
entry .edp.f.te -width 60 -textvariable titl
pack .edp.f.t -side left
pack .edp.f.te -side right
frame .edp.g
label .edp.g.w -width 20 -text "Wavelength"
entry .edp.g.we -width 20 -textvariable wav
pack  .edp.g.w -side left
pack .edp.g.we -side left 
frame .edp.h
label .edp.h.h -width 20 -text "Add zero offset"
entry .edp.h.he -width 20 -textvariable zoff
pack  .edp.h.h -side left
pack .edp.h.he -side left 
frame .edp.w
label .edp.w.t -width 20 -text "theta-eps default"
entry .edp.w.te -width 20 -textvariable dtheta
pack  .edp.w.t -side left
pack .edp.w.te -side left 
frame .edp.k
button .edp.k.b -text "Confirm changes" -command {
.edp.l configure -text ""
# check reals for wav/zoff
set gudwav [ expr $wav]
set mm ""
if { $gudwav < 0.3  | $gudwav > 10.0  }  { set mm "  Wavelength unacceptable  " } 
set nn ""
set gudzoff [ expr $zoff ]
if { $gudzoff < -10. | $gudzoff > 10. } { set nn "  Zero offset huge" }
set pp ""
set guddth [ expr $dtheta ]
if { $guddth > 1.0 } { set pp " dtheta huge" }
set err "$mm$nn$pp"
if { $err == "" } {
uphead 1
destroy .edp
}
if { $err != "" } {
.edp.l configure -text $err
} }
button .edp.k.x -text "Dismiss - no changes" -command {
#restore edit state
destroy .edp }
pack .edp.h.h -side left
pack .edp.h.he -side left
pack .edp.l -side top
pack .edp.f -side top
pack .edp.g -side top
pack .edp.h -side top
pack .edp.w -side top
pack .edp.k.b  -side left
pack .edp.k.x -side right -padx 20
pack .edp.k -side bottom -padx 20
focus .edp
grab  set .edp
tkwait window .edp
}

###################### edtpro  edit project peak file bproj.PPP etc
proc edtpro {  } {
global proj filerr fproj titl wav zoff dtheta cfont ddt min2t max2t
global edtd bproj
debug "edtpro  "
toplevel .fnm 
frame .fnm.f 
label .fnm.f.l -width 40  -text "Create/edit peak file $bproj.PPP"
label .fnm.f.lerr -width 40 -text ""
frame .fnm.f.t
text .fnm.f.t.t -width 40 -height 30 -xscrollcommand ".fnm.f.t.scrollx  set" \
-yscrollcommand ".fnm.f.t.scrolly set" -wrap none -font $cfont
scrollbar .fnm.f.t.scrolly -command ".fnm.f.t.t yview " -orient v
scrollbar .fnm.f.t.scrollx -command ".fnm.f.t.t  xview " -orient h
set fproj $bproj.PPP
set ttxt \
"title   edit this and following lines
10.1     or do copy/paste
15.0
20.1     
25.1 (peak4 2theta) 
26.1 (peak5 2theta) 
27.1 peak6 2theta 
etc   (no duplicates!)

lines containing * will be omitted in treatment

Needs 20 or more  2theta values 
in increasing sequence between $min2t and $max2t
for reliable results! "

set tt ""
if { [file exists $fproj ] } {
set ff [ open $fproj r ]
while { [ gets $ff nl ] >= 0 } {
.fnm.f.t.t insert end "$nl\n"
}
close $ff 
} else {
.fnm.f.t.t insert end $ttxt
# should never occur.......
}

button .fnm.f.s -text "Save" -width 20  -command { 
set edtd 1
set hedited " edited"
edttht }
pack .fnm.f.l -side top
pack .fnm.f.lerr -side top
pack .fnm.f.s -side bottom
pack .fnm.f.t.scrollx -side bottom -fill x
pack .fnm.f.t.scrolly -side right -fill y
pack .fnm.f.t.t -side top 
pack .fnm.f.t -side bottom
pack .fnm.f.s
pack .fnm.f
focus .fnm
grab set .fnm
tkwait window .fnm
}



################       edttht       save data to project peak file

proc edttht { } {
# some redundancy with chktht
global proj filerr fproj titl wav zoff dtheta cfont ddt min2t max2t
global edtd bproj hedited ltxt lthet badval badrge badseq
# first check values are reals between min2t and max2t degrees
debug "edttht    "
#save title line and check two thet values
set titnow  [ .fnm.f.t.t get 1.0 1.end ]
set res [ .fnm.f.t.t get 2.0 end ]
set lreal ""
set ltxt [ split $res "\n" ]

chkppp

if { $filerr != ""  } {
set errtxt "Error line#(0=OK) value/range/sequence $badval/$badrge/$badseq"
.fnm.f.lerr configure -text $errtxt -fg red
return }
if { $filerr == "" } {
set nnn [ llength $lthet ]
# writout file and update header in main panel 
if { $nnn < 20 }  { message " Only $nnn peaks read in - possible problems ahead!
\r Most programs need 20 or more" }
set ff [ open $fproj  w ]
puts $ff $titnow
set i 0
foreach th $lthet {
if { [string first * $th ] < 0 } {
set thout [ format "%9.4f" $th ]
puts $ff $thout } else {
puts $ff $th 
}
} 
close $ff
set titl $titnow
set edtd 1
set hedited " edited"
vput
# save status for future
uphead 1
#.main panel updated with title.
destroy .fnm
 }
 }

########################### endtsk end subtask
proc endtsk { } {
global pruns pexenam pform logtxt killd
debug "endtsk   "
# read procid from file crsubpid; if found kill it
set killd 0
set rpid ""
set tid ""
set tpid 0
catch {
set pf [ open "crsubpid" r ]
gets $pf rpid
close $pf
}
if { $rpid != "" } {
set tid [ string range $rpid 0 1 ]
set tpid [ string range $rpid 3 11]
set logtxt "$pexenam forced termination"
set killd 1
logit
set pf [ open "crrun" w ]
puts $pf "$tid  terminated abruptly"
close $pf
catch {
if { $pform == "unix" } {
set endsub [ exec kill -9 $tpid ]
}
if { $pform != "unix" } {
set endsub [ exec taskkill /F /pid $tpid ] 
}
}
}
}

########################### ftini  sets up font
proc ftini {} {
global homedir crfont setscal cfont fnames fsizes 
# read current scaling if any from .CRfont file
debug "ftini   "
set setscal 100%
set crfont $homedir/.CRfont
set ff [ file exists $crfont ]
if { $ff != 1 } {
set ou  [ open $crfont w ]
puts $ou $setscal 
close $ou } else {
set ou [ open $crfont ]
gets $ou setscal 
}
# make list of fonts and default sizes for resets
set fnames [ font names ]
foreach fn $fnames {
lappend fsizes [ font configure $fn -size ]
#keep list of original font sizes
}
}


########################### ftset  rescale/reset font sizes
proc ftset {} {
global crfont setscal cfont crfont
debug "ftset   "
catch { destroy .sm }
toplevel .sm
grab set .sm
focus .sm
wm title .sm "Setting scaling for font size"
set newscal $setscal
set scall [ list 75% 100% 125% 150% 200% ]
frame .sm.t
frame .sm.b
button .sm.t.done -text "Save current" -command {
#update file .CRfont
set ff [ open $crfont w ]
puts $ff $setscal
close $ff
ftupt
destroy .sm
return }
button .sm.t.revert -text "Revert to 100%" -command {
set setscal 100%
ftupt
#update file
set ff [ open $crfont w ]
puts $ff $setscal
close $ff
ftupt
destroy .sm
return }

pack .sm.t.done -side left
pack .sm.t.revert -side left
pack .sm.t -side top
foreach rb $scall {
radiobutton .sm.b.$rb -variable setscal -text $rb -value $rb -command { 
ftupt
}

pack .sm.b.$rb -side left
incr b

}
pack .sm.b
}



########################### ftupt

proc ftupt {} {
global crfonts setscal cfont fnames fsizes pform
debug "ftupt  "
set ss [ string trimright $setscal % ]
set cour 14
if { $pform != "unix" } {set cour 16 }
set fcourier [ expr int ( $cour * $ss /100.) ]
set cfont  "-adobe-courier-medium-r-normal--$fcourier-*-*-*-*-*-*-*-*"  
# now scale explicitly each built-in Tk font
set b 0
foreach fn  $fnames {
set s [ lindex $fsizes $b ]
set newscal [ expr int ( $ss * $s / 100. ) ]
font configure $fn -size $newscal 
incr b
}}


########################### ini  initialise routine , set cwd
proc ini { } {
global homedir atend version cfdir pform titl proj zoff wav env resbut newp
global lid lexe lnam llid work pdir sdir proroot header  titl sect newroot
global sheader sheader1 sheader0 cfont ddt dtheta logtxt 
global pexenam pgexe killd fproj
global crversion vrw bproj edtd hedited
debug "ini    "
set lid [ list TP IT FJ TR KL DV LZ LS ]
set llid [ list tp it fj tr kl dv lz ls]
set lexe [ list taup32 ito13 fjzn621 treor90 kohl620 dicvol91 lzon622 losh62 ]
set lnam [ list TAUP ITO FJZN TREOR KOHL DICVOL LZON LOSH ]
set killd 0
set sect 0
set atend 0
set version 1.0
set titl ""
set zoff 0
set wav 1.54036
set dtheta 0.03
set proj ""
set pproj ""
set fproj ""
set resbut 0
set newp ""
set proroot $homedir/.CRroot
set newroot $homedir
set titl "** Not Set ** "
set work $homedir/cr2020
set bproj ""
set hedited ""
set crversion 1
set edtd 0
if { $sect == 0 } {
# first time round
# timing - use clicks since not all tcl have milliseconds
set t0 [ clock clicks ]
after 1000
set t1 [ clock clicks ]
set sect [ expr $t1 - $t0 ]
}
# ticks per second sect 

#  find existing project root proroot  from ~user/.CRroot
#  file  >>> proroot/cr2020/.CRproj has names of  current project or ""

set ff [ file exists $proroot ] 
## always start first time with default dir and default empty project name
if { $ff != 1 } {
debug " ini  overwriting proroot  with $work "
set ou [ open $proroot w ]
puts $ou $work
close $ou
}

set in [ open $proroot r ]
gets $in twork
close $in
debug "ini twork   twork"
if { [ file isdirectory $twork ] != 1 } {
message " Invalid directory in .CRroot '$twork' \n  Reset to default $work \n
Use settings/change directory  to modify this "
set ou [ open $proroot w ]
puts $ou $work
close $ou
file mkdir $work
set pnamf $work/.CRproj
debug "ini pnamf $pnamf"
set ou [ open $pnamf w ]
puts $ou ""
close $ou
} 

# root file found...


if { $ff == 1 } {
debug "ini  proroot is $proroot"
set in [ open $proroot r ]
gets $in newr
set newroot [ file dirname $newr ]
debug "ini found newroot $newroot " 
close $in
# now look for any current project
set  work $newroot/cr2020
catch {
set pnamf $work/.CRproj
set in [ open $pnamf r ]
gets $in bproj
# overwrites "" base project name
close $in 
}
debug "ini current project is '$bproj' "
# bproj is blank if not found
if { $bproj != "" } {
# now check project exists
set fpp [ file isdirectory $work/$bproj ]
set titl "title missing"
if { [ file isdirectory $work/$bproj ] == 1 } {
set dfxred 0
catch {
# now read .dfx file for current title etc otherwise use defaults
set in [ open $work/$bproj/crdfx ]
gets $in titli
if { $titli != "" } { set titl $titli }
debug "ini titl is $titl   titli is  ... $titli" 
gets $in zoffi
if { $zoffi != "" } { set zoff $zoffi }
gets $in dthetai
if { $dthetai != "" } { set dtheta $dtheti }
gets $in wavi
if { $wavi != "" } { set wav $wavi }
close $in
set dfxred 1
}
if { $dfxred != 1 } {
set fout [ open $work/$bproj/crdfx w ]
puts $fout $titl
puts $fout $zoff
puts $fout $dtheta
puts $fout $wav
close $fout
}
}
#
}
} else {
# new blank project
set bproj ""
set in [ open $work/.CRproj w ]
puts $in ""
close $in
#### clear bum project
}


 
set header "Current work directory is $work \n \
Current project is $bproj version $crversion\n  $hedited \
Default title is $titl \n \
Zero offset is $zoff \n \
Dtheta is $dtheta \n \
Wavelength is $wav \n"
set sheader ""
set sheader0 "Running summary of all solutions logged so far for this project name\n,
sorted in descending order of I20 then Merit (as defined by each program)"

set sheader1 "I20  Merit    Volume   V/V1 BL IndexProg    Date     \
Time    Pedig     a        b        c   \
alpha    beta   gamma     Q(A)      Q(B)      Q(C)     \
Q(D)      Q(E)      Q(F)   Description\n"
	       

#either open known file or await file command - which ????
set newp $bproj
if { $bproj != "" } {
debug "ini work is $work  proj is $bproj " 
set pdir $work/$bproj
cd $pdir
#now in work directory - get current version
vget
set proj $bproj\_$crversion 
if { $edtd != 0 } { set hedited " edited" }
# get current version and edit status
set proj $bproj\_$crversion
set logtxt "Resuming project $work/$bproj  version $crversion proj $proj"
logit
}
} 


####################  logit       appends to log file proj.log
proc logit { } {
global ttssdd bproj logtxt
debug "logit  $logtxt "
tdate
catch {
set ff [ open $bproj.log  a ]
set lin "$ttssdd $logtxt"
puts $ff $lin
close $ff
}
}

####################  maksum      concat=> file, sort, update vratio, run clepage
proc maksum { } {
global proj sdir pform logtxt sect
debug "maksum sdir $sdir"
# need to delete .CLO .CLM .SUM files and disable display buttons for LEPAGE
set ff [ file delete $proj.SUM ]
set ff [ file delete $proj.CLO ]
set ff [ file delete $proj.CLM ]
# disable view buttons for clepag
.all.sumo configure -state disabled
.all.summ configure -state disabled
# make list of idM files present in this directory
set fils [ glob -nocomplain  $proj.*M ]
if { [llength $fils ] <= 0 } {
#.main.tt configure -text "No summaries available from indexing programs"
texinfo "red" "No summaries available from indexing programs"
return
}
list fcat
foreach fin $fils {
set fi [ open $fin r ]
while {  [ set lin [ gets $fi  ] ] > 0 } {
lappend fcat $lin
} 
set fff $fi
close $fi
}
# now sort file and modify volume ratio


#######  sort list
set sfcat [ lsort -ascii -decreasing $fcat ]

#######  modify vratio
set logtxt "concatenate summary files, sort, update vratio, Running clepage14"
logit
set fsum [ open $proj.SUM w ]
set lin [ lindex $sfcat 1 ]
####### get volume from first line
set v1 [ string range $lin 10 21]
set hhh [ llength $sfcat]
set ii 0
foreach lin $sfcat {
incr ii
#if { $ii >= 10 } { break }
set vv [ string range $lin 10 21 ]
set vrr [ expr $vv / $v1 ]
set vrrf [ format "%6.2f" $vrr  ]
set lstart [ string range $lin 0 20 ]
set lend [ string range $lin 27 end ]
puts $fsum $lstart$vrrf$lend
}
close $fsum
set pgmsuf ""
if { $pform != "unix" } { set pgmsuf ".exe" }
# originally use external sort and vratio
#set vrr $sdir/vratio12$pgmsuf
#set res [ exec sort <summed.ZZZ >summed.RRR ]
#set res [ exec $vrr summed.RRR summed.ZZZ ]
set clep $sdir/clepag14$pgmsuf

texinfo \#006400 "Running $clep"
update
set tzero [ clock clicks]
set res [ exec $clep $proj > nul ]
set logtext "finish Lepage summary"
#.main.tt configure -text "Finished $clep"
set tnow [ clock clicks]
set deltr [ expr ( $tnow - $tzero ) *1.0  /  $sect  ]
set delt [ format "%7.2f" $deltr ]
set logtxt "Ending clepage14 after $delt seconds"
texinfo "blue" "Finished $clep after $delt seconds"
logit
chkbut
}

####################  message     ddt window
proc message {  W  } {
toplevel .msg
frame .msg.f
label .msg.f.label -text " ... $W ..." -fg blue
button .msg.f.but -text "OK" -command {destroy .msg }
pack .msg.f.label  .msg.f.but
pack .msg.f
focus .msg
grab set .msg
tkwait window .msg
}

###################### newcrrt  creates new working root dir
proc newcrrt { } {
global homedir  newroot work savroot bproj fwrite titl zoff wav dtheta ddt proj
debug "newcrrt  "
set savroot $newroot
set newroot $homedir
set bproj ""
set proj ""
toplevel .nm
focus .nm
grab set .nm
# initialise title and defaults
set wav 1.54036
set titl "** not set **"
set zoff 0
set dtheta 0.03
frame .nm.f -height 10 -width 40 
label .nm.f.t -text " "
label .nm.f.lb -text "Give path to main cr2020 newroot directory \n\n (NOTE - this directory must exist)" -fg blue
# newroot ust be global for textvariable to newroot
entry .nm.f.en -text "$newroot" -textvariable newroot  -width 50
button .nm.f.but -text "OK" -command {
# check presence of OK workfile directory
chkwrt $newroot
debug " newcrrt newroot fwrite $newroot  $fwrite" 
set err 0
if { $fwrite != 1 } {
.nm.f.t configure -text "$newroot is an invalid directory path for new workfiles " \
-fg "red"
return 
}
#  reset title, wav, zer
# existing work directory
if { [file isdirectory $newroot/cr2020 ] == 1 } {
set proroot $homedir/.CRroot
# reset current project blank
set ff [ open $proroot w ]
debug " current project root proroot : $proroot   newroot $newroot/cr2020"
puts $ff "$newroot/cr2020"
puts $ff ""
close $ff 
set pnamf "$newroot/cr2020/.CRproj"
set bproj ""
set proj ""
set ff [ open $pnamf w ]
puts $ff $bproj 
close $ff  
debug " pnamf $pnamf now contains bproj $bproj  and newroot is $newroot"  
debug "newcrrt newroot is $newroot" 
texinfo "red" "Please use 'File' to open new or resume existing project "
# need to update header, set work etc
set work $newroot/cr2020
uphead 0
destroy .nm 
}

# no work directory is here - make a cr2020 directory
file mkdir $newroot/cr2020 
if { [file isdirectory $newroot/cr2020 ] == 0 } {
.nm.f.t configure -text "$newroot is an invalid directory path for new workfiles " \
-fg "red"
set err 1
return
} 
if {$err == 0 } {
set err 1
catch {
file mkdir $newroot/cr2020
}  }
if { [file isdirectory $newroot/cr2020 ] ==1 } {
set err 0 }
debug " err is $err; newroot/cr2020  dir is dir $newroot/cr2020"
if { $err == 1 } {
.nm.f.t configure -text "Couldn't create $newroot/cr2020 " -fg red
set newroot $savroot
debug "newcrrt newroot is $newroot" 
return
}
#all seems well - set proj empty
if {$err == 0 } {
# save this info in $homedir/.CRroot
set proroot $homedir/.CRroot
debug " proroot $proroot is  homedir/.crroot $homedir/.CRroot"
set ff [ open $proroot w ]
puts $ff "$newroot/cr2020"
puts $ff ""
close $ff
set work $newroot/cr2020
debug " work is $work for update"
uphead 0

destroy .nm }
}   

pack .nm.f.t
pack .nm.f.lb .nm.f.en
pack .nm.f.but
pack .nm.f
tkwait window .nm

} 

###################### newpron  gets newprojectname
proc newpron { } {
# NOTE most here follows from accepting filename  i.e. from a button command hence
# data in use must all be on global
# newpro calls newprof calls chktht calls chkppp
global proj work proje fproj titl zoff wav dtheta proroot pdir lthet ttl filerr newttl emptyttl
global logtxt bproj edtd hedited crversion
debug "newpron "
toplevel .ppp
wm title .ppp "Create new Project file"
grab set .ppp
focus .ppp
set proje "new"
set bproj $proje
set emptyttl "** No Title Set ** "
set newttl $emptyttl
set lthet ""
label .ppp.m -text ""
frame .ppp.n
label .ppp.n.l -text "Project name (max. 20 chars)"
entry .ppp.n.e -textvariable proje  -width 12
frame .ppp.p
label .ppp.p.t -text "Title"
entry .ppp.p.e -textvariable newttl -width 40
button .ppp.d -text "OK - Set this project name" -command {
.ppp.m configure -text ""
if  { [ file isdirectory $work/$proje ] == 1 } {
.ppp.m configure  -text "This project exists already - must use 'resume project'" -fg red
} elseif { $newttl == $emptyttl } {
.ppp.m configure -text "Please set Title " -fg red
} elseif { [ string length [ string trim $proje ] ] > 20  } {
.ppp.m configure -text "Project name too long" -fg red
}  else {
set bproj [ string trim $proje ]
set filerr ""

newprof


# newprof calls chkthet to fill lthet
# by this stage twotheta values should be ok ready in lthet to create project dir etc.
set nthetin [llength $lthet ]
# if all is ok thencreate director and write out data
debug "newpron filerr $filerr nthetin $nthetin"
if { $filerr == "" && $nthetin >0 } {
set ff [ file mkdir $work/$proje ]
.ppp.m configure -text "Creating directory $work/$proje "
set bproj $proje 
#
#

# shouldnt fproj be set here?  to $proj.PPP

#
#
set pdir $work/$bproj
#####     set current directory to new proj directory and copy para file
cd $pdir
catch {
set ff [ file copy -force "$sdir/cr2020.dfp" "crdfp"]
}
# write out data file
set nnn [ llength $lthet ]
# writout file and update header in main panel 
set titnow $newttl
if { $nnn < 20 }  { message " Only $nnn peaks read in - possible problems ahead!
\r Most programs need 20 or more" }
set ff [ open $fproj  w ]
puts $ff $newttl
foreach th $lthet {
if { [ string first * $th ] < 0 } {
set thout [ format "%9.4f" $th ]
# simply copy any line with *
puts $ff $thout   }  else {
puts $ff $th
}}
 
close $ff
set titl $newttl
if { $wav == "" | $wav < 0.3 } { set wav 1.54036 }
if { $zoff == "" | $zoff  > 10. | $zoff < 10. } { set zoff 0.0 }
if { $dtheta == "" | $dtheta > 1.0  | $dtheta < 0.005 } { set dtheta 0.03 }
#.main panel updated with title.
# now update current work/proj file...
# default file is cfwork
set hh [ open $proroot w ]
puts $hh $work
close $hh 
set pnamf $work/.CRproj
set hh [ open $pnamf w ]
puts $hh $bproj
close $hh
set logtxt "Creating project $bproj "
set crversion 1
set edtd 1
set proj "$bproj\_$crversion"
logit
uphead 1
chkbut
texinfo "blue"  "New project $bproj created"
}  else  {
texinfo "orange" "Preparation of project $proje cancelled"
}
destroy .ppp
 } }
pack .ppp.d -side bottom
pack .ppp.n.l -side left
pack .ppp.n.e -side right -expand 1
pack .ppp.p.t -side left
pack .ppp.p.e -side right -expand 1
pack .ppp.m
pack .ppp.n 
pack .ppp.p
}

####################### newprof  create new project peak file proj.PPP etc
proc newprof {  } {
global proj filerr fproj titl wav zoff dtheta cfont ddt ttl intab seldon setit
global lthet newtit newttl proj defcol min2t max2t
global edtd hedited bproj
debug "newprof  "
set inlis ""
set selcl ""
set defcol 1
set edtd 1
set hedited " edited"
vput
toplevel .nwp 
wm title .nwp "Create Peak file $bproj.PPP "
frame .nwp.f 
label .nwp.f.l -width 50  -text "Create  peak file $bproj.PPP \r
Copy column of 2Theta or table into window below and
select desired column
then SAVE"
label .nwp.f.lerr -width 40 -text ""
frame .nwp.f.col
label .nwp.f.col.l -width 15 -text "Use column # "
entry .nwp.f.col.e -width 3 -text "1" -textvariable defcol
frame .nwp.f.til
label .nwp.f.til.l -width 40  -text  "Current Title: $newttl"
frame .nwp.f.t -bg gray -width 45
text .nwp.f.t.t -width 60 -height 30 -xscrollcommand ".nwp.f.t.scrollx  set" \
-yscrollcommand ".nwp.f.t.scrolly set" -wrap none -font $cfont
scrollbar .nwp.f.t.scrolly -command ".nwp.f.t.t yview " -orient v
scrollbar .nwp.f.t.scrollx -command ".nwp.f.t.t  xview " -orient h
frame .nwp.f.b
button .nwp.f.b.s -text "Save" -width 12  -command { chktht } -state disabled
button .nwp.f.b.sel -text "Select column" -command { npslcl }
button .nwp.f.b.bak -text "Undo selection" -command { npsbak } -state disabled
pack .nwp.f.l -side top
pack .nwp.f.lerr -side top
pack .nwp.f.til.l -side left
pack .nwp.f.til -side top
pack .nwp.f.col.l  -side left 
pack .nwp.f.col.e -side left
pack .nwp.f.col -side top
pack .nwp.f.t.scrollx -side bottom -fill x
pack .nwp.f.t.scrolly -side right -fill y
pack .nwp.f.t.t -side top -expand 1
pack .nwp.f.t -side bottom
pack .nwp.f.b.sel -side left  -padx 5
pack .nwp.f.b.bak -side left  -padx 5
pack .nwp.f.b.s -side left
pack .nwp.f.b -side bottom
pack .nwp.f
focus .nwp
grab set .nwp
set fproj $bproj.PPP
set ttxt "
This is an editable text window

Please replace this note using copy/paste 
(NB use CTRL/V to paste in this window from clipboard)
of a single column of two theta values
or, alternatively, a table containing a column of
two theta values.  Column separators may include
white space, commas, or tabs.  You can then select
the required data column and save it 
Two theta values must be sequential 
in ascending order and be
between $min2t and $max2t degrees"


set tt ""
.nwp.f.t.t insert end $ttxt


tkwait window .nwp
}

################      npslcl        selects column from data table
proc npslcl {} {
global defcol intab newttl
debug "npslcl   "
.nwp.f.lerr configure -text ""
set intab [ .nwp.f.t.t get 1.0 end ]
#conserve input to allow undo
set lread ""
set ltxt [ split $intab "\n" ]
# split each line and check line lengths
set min 1000
set max 0
set err 0
set outtab ""
set usecol ""
foreach lin $ltxt {
#remove white space
set usecol ""
set bits [ split $lin ]
set colno 0
foreach ele $bits {
if { $ele != "" } { 
incr colno
lappend usecol $ele
if { $colno == $defcol } { lappend outtab $ele }
}
}
set llen [ llength $usecol  ]
if { $max < $llen } { set max $llen}
if { $llen > 0 && $min > $llen } { set min $llen}
lappend ttab $usecol
}
set outtabl [ llength $outtab]
if { $min != $max } {
.nwp.f.lerr configure  -text "mismatch min/max columns $min / $max " -fg red
return 
}
if { $defcol > $min } {
.nwp.f.lerr configure -text "only $min columns - can't use $defcol " -fg red
return
}
# now rewrite
.nwp.f.t.t delete 1.0 end
.nwp.f.t.t insert end "$newttl\n"
foreach ele $outtab {
.nwp.f.t.t insert end $ele\n
}
.nwp.f.b.bak configure -state normal 
.nwp.f.b.s configure -state normal
}


################      npsbak        replace column into data table
proc npsbak {} {
global intab seldon
debug "npsbak   "
# delete .nwp.f.t.t  and replace with intab
.nwp.f.t.t delete 1.0 end
.nwp.f.t.t insert end $intab
.nwp.f.b.s configure -state disabled
set seldon 0
}
################### resproj resume existing project
proc resproj { } {
global proj fproj work newp okres resbut ddt pross resok bproj
debug "resproj  "
#  may change workdir/projdir - need to resave these
# open project and treat as new with edit of peak file...
# update header in main
# indicate that button was used to initiate resume (not from ini)
set resbut 1
toplevel .rpj
focus .rpj
grab set .rpj
set now [ pwd ]
debug "xx$ddt resproj now is $now cwd is $work" 
cd $work
set pros [ glob -nocomplain * ]
#set pros [ list 1asd 2asd 3asd 4asd 5asd 6assd 7asd ]
set pross [ lsort $pros ]
set tle "Projects in workdirectory $work "
debug "resproj pross is $pross" 
label .rpj.t -text "Current Projects"
label .rpj.e -text "work is $work proj is $bproj"
listbox .rpj.l -width 25 -height 20  -yscrollcommand ".rpj.scroll set" \
-background white -selectmode single
scrollbar .rpj.scroll -command ".rpj.l yview"
foreach fil $pross {
#puts $fil
.rpj.l insert end $fil
}
button .rpj.v -text "Use selected project" -command {
set fno [ .rpj.l curselection ]
set fil [ lindex $pross $fno ]
chkres $fil
update
if { $resok == 0 } { destroy .rpj}
}
grid .rpj.t
grid .rpj.e
grid  .rpj.l .rpj.scroll -sticky news
grid .rpj.v
tkwait window .rpj
}


################### runpgm     runs specific indexing program
proc runpgm { pnow } {
global llid lid lnam lexe pform sdir proj ttssdd tzero delt logtxt sect
global pexenam pexe pgexe killd psta
debug "runpgm   proj $proj  "
# perhaps the most important part using proj rather than bproj
# to ensure all output files match with same version
debug "runpgm "
if { $proj == "" } {
texinfo "red" "Please use 'File' to open new or resume existing project "
return }
# need to delete .CLO .CLM .SUM files and disable display buttons for LEPAGE
set ff [ file delete $proj.SUM ]
set ff [ file delete $proj.CLO ]
set ff [ file delete $proj.CLM ]
# disable view buttons for clepag
.all.sumo configure -state disabled
.all.summ configure -state disabled
# no longer valid sum files after running options
set pgmsuf ""
if { $pform != "unix" } { set pgmsuf ".exe" }
set pg [lindex $lexe $pnow ]
set pexenam $pg
set id [lindex $lid $pnow ]
# full exe filename
set pgexe $sdir/$pg$pgmsuf
set psta "$sdir/psta$pgmsuf"
# summary is .idM  long listing is idD - all have idO listing
append ids $id  M
append ido $id O
append ide $id E
set logtxt "Running $pg"
logit
catch {
set ff [ file delete -force $proj.$ids ]
set ff [ file delete -force $proj.$ido ] }
#################chkbut
# reset file display status
set logtxt "starting $pg"
set green #006400
#this is dark green
texinfo \#006400 $logtxt

update idletasks
set tzero [ clock clicks ]
#testing after 500
set errfil "$proj.$ide"
set pexe $pgexe
set psecs 0
set pruns 0
set killd 0

# now run task and clock

runpxe

# special cases to run for external log summaries...itlog(1) dvlog(5) trlog(3)  etc

if { $pg == "ito13" } { 
set lg "$sdir/itlog$pgmsuf"
set go [ exec $lg $proj ] }
if { $pg == "treor90" } { 
set lg "$sdir/trlog21$pgmsuf"
set go [ exec $lg $proj  ] }
if { $pg == "dicvol91" } { 
set lg "$sdir/dvlog"
set go [ exec $lg $proj ] }
#enable view results if found.
set tnow [ clock clicks]
set deltr [ expr ( $tnow - $tzero ) *1.0  /  $sect  ]
set delt [ format "%7.2f" $deltr ]
#set logtxt "$pg finished"
set logtxt "Ending $pg after $delt seconds"

# termination messages

catch {
set ff [ open "crrun" r ]
gets $ff linin
close $ff
# get termination message from background program
set trlinin [ string range $linin 3 end]
set logtxt "$pg finished $trlinin"
}
if { $killd == 1 } { set logtxt "Interrupted $pg after $delt seconds" }
texinfo "blue" $logtxt
logit
# now remove files with zero content
set unsorf ""
catch {
set unsorf [ glob -nocomplain  $proj.$id* ]
}
if { [llength $unsorf ] != 0  } {
foreach fl $unsorf {
set ff [ open $fl r ]
set nf [ gets $ff lin ]
close $ff
if { $nf == -1 } { file delete -force $fl }
}  }
chkbut
}


################### runpxe runs and updates clock
proc runpxe { } {
global pexenam proj pgexe killd pform psta logit logtxt
debug "runpxe   "
# task control endtsk stops program and sets killd 
# first create monitor window .runp
toplevel .runp 
label .runp.lb -width 30 -height 3 -text "$pexenam starting" 
button .runp.b -width 25 -foreground "red" -text "INTERRUPT $pexenam" -command { endtsk }
grab .runp
pack .runp.lb
pack .runp.b
wm protocol .runp  WM_DELETE_WINDOW { 
 endtsk 
   }
set ttt 1
set tim 0
set killd 0
set runng 0
set status starting
set rtim 0
debug "runpxe proj is $proj"
# first check no edits to data and version changes have occurred
vnew
set txt " $pexenam  $status - $rtim secs "
.runp.lb configure -text $txt
update
after 500
# bring up cancel window before actually running program
# first delete pid file
catch { set w [file delete -force "crrun" ]
set w [file delete -force "crsubpid"] }

# run exe
# WINDOWS
# tcl-background cpu intensive tasks hog all cpu time until finished
# necessary to launch via windows CREATE_PROCESS call hich creates a
# distinct independent process with normal priority
# for OSX and unix this does not happen - cpu is shared around
# behaviour is identical for WINDOWS tcl v8.5.9 and 8.6.13

set w -1
catch {
if { $pform != "unix" } {
set w [ exec $psta "$pgexe $proj" &    ]
}
if { $pform == "unix" } {
set w [ exec $pgexe $proj & ]
}
}
# always short pause to show interrupt window
while { $ttt == 1 } {
after 100
incr tim

if { $status == "starting"  } { 
set runng [ file exist "crsubpid" ]
if { $runng == 1 } { 
set status "running" 
set logtxt "running $pgexe"
set green #006400
#this is dark green
texinfo \#006400 $logtxt
}
}
set rtim [ expr $tim/10. ]
set txt " $pexenam    $status - $rtim secs "
.runp.lb configure -text $txt
update
# terminates with kill command from interrupt or creation of crrun file
set endl [ file exists crrun ]
if { $endl == 1 } { set ttt 0}
if { $killd == 1 } { set ttt 0}
}
destroy .runp
}


################### setupallp  setup program buttons
proc setupallp { } {
global lexe lid lnam llid pcur lbutt lext lbutt lext proj sdir pform
debug "setupallp  "
frame .ind -width 60
set np [ llength $lid ]
for { set nn 0 } { $nn < $np } { incr nn } {
set idd [ lindex $lid $nn ]
set pg [ lindex $lexe $nn ]
set pg $sdir/$pg
if { $pform != "unix" } { set pg  $pg.exe }
set sstate "disabled"
if { [ file executable $pg ] } { set sstate "normal" }
frame .ind.$nn
set nam [lindex $lnam $nn]
button .ind.$nn.p -text $nam -width 20 -command "runpgm $nn" -state $sstate
button .ind.$nn.s -text "show summary" -width 20 -command "vusout $idd M" -state disabled
lappend lext "$idd\M"
lappend lbutt ".ind.$nn.s" 
button .ind.$nn.o -text "show output" -width 20 -command "vusout $idd O" -state disabled
lappend lext "$idd\O"
lappend lbutt ".ind.$nn.o" 
pack .ind.$nn.p .ind.$nn.s .ind.$nn.o -side left
pack .ind.$nn -side top
}
#puts $lext
#puts $lbutt
pack .ind
frame .all -bg white
button .all.sum -text "Summarise/Lepage" -width 20 -height 2 -command "maksum"
button .all.sumo -text "show full Lepage" -width 20 -height 2 -command "shosum O" -state disabled
button .all.summ -text "show summary" -width 20 -height 2 -state disabled -command "shosum M"
pack .all.sum .all.summ .all.sumo -side left
pack .all
}

################### shofil   show list of files in project
proc shofil { } {
global bproj cfont vufil fils atend pform
debug "shofil  "
if { $bproj == "" } {
texinfo "red" "No project set - use file command" 
return  }
set atend 1
set unsorf ""
catch {
set unsorf [ glob -nocomplain  $bproj*.* ]
}
if { [llength $unsorf ] == 0  } { 
texinfo "red" "No files in project $bproj"
return 
}
# fils is list of files
# sort list
set fils [ lsort $unsorf ]
toplevel .fhp
wm title .fhp "Project files"
grab set .fhp
focus .fhp
button .fhp.b -text "Dismiss" -command {
destroy .fhp}
set titl "Current files in project $bproj "
listbox .fhp.l -width 30 -height 20 -font $cfont -yscrollcommand ".fhp.scroll set" \
-background white -selectmode single
scrollbar .fhp.scroll -command ".fhp.l yview"

.fhp.l insert end $titl
foreach fil $fils {
#puts $fil
.fhp.l insert end $fil
}
button .fhp.v -text "View selected file" -command {
set fno [ .fhp.l curselection ]
incr fno -1
set vufil [ lindex $fils $fno ]
viewfile
}

grid  .fhp.l .fhp.scroll -sticky news
grid .fhp.v
tkwait window .fhp
}


################### shopro   show list of projects
proc shopro { } {
global work
debug "shopro "
set now [ pwd ]
cd $work
set pros ""
catch {
set pros [ glob -nocomplain * ]
}
cd $now
if { [llength $pros ]  == 0  } {
texinfo "red" "No previous projects - use file command"
return
}
# pros is list of files
toplevel .shp
grab set .shp
focus .shp
button .shp.b -text "Dismiss" -command {
destroy .shp}
label .shp.t -text "Current projects in $work "
set dir ""
foreach f $pros {
set dir "$dir \n $f "
}
label .shp.d -text $dir
pack .shp.t -side top
pack .shp.d -side top
pack .shp.b -side top
tkwait window .shp
}

###################  shosum  sho Lepage summaries
proc shosum { typ } {
global vufil proj atend
debug "shosum "
set atend 1
set vufil "$proj.CL$typ"
viewfile
}


########################### texinfo  writes texinfo bar
proc texinfo { col txt } {
.main.tt configure -text $txt -foreground "$col"
}


###################  tdate  time and date
proc tdate {} {
global ttdd ttssdd
set ttdd [clock format [clock seconds] -format "%H:%M   %d/%m/%y" ]
set ttssdd [clock format [clock seconds] -format "%H:%M:%S  %d/%m/%y" ]
}


################### uphead   update header in main panel; update .cr2020dfx
proc uphead { upfil } {
global titl wav zoff bproj header work dtheta wmol
global crversion hedited edtd
debug "uphead "
#label  .main.cur -text $header
set header "Current work directory is $work \n \
Current project is $bproj version $crversion $hedited\n \
Default title is $titl \n \
Zero offset is $zoff \n \
Dtheta is $dtheta \n \
Wavelength is $wav \n"
.main.cur configure -text $header
# update crdfx file
if { $upfil != 0 } {
set ff [ open "crdfx" w ]
puts $ff $titl
puts $ff $zoff
puts $ff $dtheta
puts $ff $wav
puts $ff 0.
close $ff
set edtd 1
set hedited " edited"
vput
}
}


################### viewfile  starts generic file viewer
proc viewfile {} {
global vufil cfont atend sheader sheader1
debug "viewfile "
catch { destroy .lt }
toplevel .lt
grab set  .lt
focus .lt
text .lt.tb  -width 120 -height 30 -font $cfont -xscrollcommand \
".lt.scrollx set" -yscrollcommand ".lt.scrolly set"  -wrap none -font $cfont
scrollbar .lt.scrolly -command " .lt.tb yview " -orient v
scrollbar .lt.scrollx -command " .lt.tb xview " -orient h
wm title .lt $vufil
tk scaling  2
set found  [ file exists $vufil ]
set ppwd [ pwd ]
if { [catch { open $vufil r } inf ] == 0  } {
if { $sheader != "" } {
.lt.tb insert end "$sheader1\n" }
while { [gets $inf line ] >= 0 } {
.lt.tb insert end "$line\n"
#puts $line
}
close $inf
#  view logfile from end
if { $atend == 0 } {.lt.tb yview end }
}
#frame .lt.bbb
button .lt.but -text "Dismiss" -command { destroy .lt}
grid .lt.tb .lt.scrolly -sticky news
grid .lt.scrollx -sticky ew
grid .lt.but
grid rowconfigure .lt 0 -weight 1
grid columnconfigure .lt 0 -weight 1 
tkwait window .lt
}


##################### vget    gets current version and edit status
proc vget {  } {
debug "vget  "
# only called in ini
global crversion edtd hedited 
set hedited ""
set edtd 0
# now check for current version
if { [ file exists pversion ] == 1 } {
set ff [ open pversion r ]
gets $ff crversion
gets $ff edtd 
if { $edtd == 1 } { set hedited " edited" }
close $ff 
return }
# absent - create 
set crversion  1
set edtd 1
set hedited " edited"
set ff [ open pversion w ]
puts $ff $crversion
puts $ff $edtd
close $ff 
}

##################### vput    puts current version and edit status
proc vput {  } {
global crversion edtd
debug "vput  "
set ff [ open pversion w ]
puts $ff $crversion
puts $ff $edtd
close $ff 
}

##################### vfil    creates current versions of input data files
# Note the .DFX and .DFP are reference copies only 
# the actual files used by fortran programs are cr2020.dfx and cr2020.dfp
proc vfil {  } {
global crversion edtd proj bproj nuse nall hedited titl
debug "vfil  "

#### update DFP file
set a [ file copy  -force crdfp $proj.DFP  ]
set a [ file copy -force crdfp cr2020.dfp  ]
# for PPP files copy data omitting starred lines
set ffin $bproj.PPP
set ffout $proj.PPP
set fin [ open $ffin r ]
set fout [ open $ffout w ]
set sss [ gets $fin tlin ]
set inppp ""
set nall 0
set nuse 0
set sss [ gets $fin linin ]
while { [ gets $fin linin ] >=0 } { 
incr nall
if { [ string first "*" $linin ] < 0 } {
# strip out commented data points
incr nuse
lappend inppp $linin
} }
close $fin
# output stripped data file - modified title first
set outtl "$nuse/$nall v$crversion:$tlin"
puts $fout $tlin
debug "vfil $outtl"
foreach peak $inppp {
puts $fout $peak }
close $fout
#### now cr2020.dfx file note first line is title which is updated 
set ffin crdfx
set ldfx ""
set fin  [ open $ffin r]
gets $fin linin
set titlo "$nuse/$nall v$crversion:$linin"
while { [ gets  $fin linin ] >= 0 } {
lappend ldfx $linin
 }
close $fin
set lll [ llength $ldfx ]
set ffout $proj.DFX
set fout [ open $ffout w ]
puts $fout $titlo
foreach linin $ldfx { 
puts $fout $linin }
close $fout
set a [ file copy -force $proj.DFX cr2020.dfx  ]
# for PPP files copy data omitting starred lines
# fisrt preerve original file with starred data lines
set a [ file copy -force $bproj.PPP $proj.PIN ]
set ffin $bproj.PPP
set ffout $proj.PPP
set fin [ open $ffin r ]
set fout [ open $ffout w ]
set sss [ gets $fin tlin ]
set inppp ""
set nall 0
set nuse 0
while { [ gets $fin linin ] >=0 } { 
incr nall
if { [ string first "*" $linin ] < 0 } {
# strip out commented data points
incr nuse
lappend inppp $linin
} }
close $fin
# output stripped data file - modified title first
set outtl "$nuse/$nall v$crversion:$tlin"
puts $fout $tlin
debug "vfil $outtl"
foreach peak $inppp {
incr nuse
puts $fout $peak }
close $fout
set edtd 0
set hedited ""
}

##################### vnew    creates new version of input data files
proc vnew {  } {
global crversion edtd proj bproj hedited logtxt
set fex [ file exists $proj.PPP ]
# most commonly new version already established.
debug "vnew  fex $proj.PPP : $fex   edtd: $edtd "
if { $fex  > 0 && $edtd == 0 } { 
return }
# no initial version 1
if { $fex == 0 }  {
vfil
set edtd 0
set hedited ""
vput
uphead 0
}
# version  already  exists but has been updated - move to next version
if { [ file exists $proj.PPP ] == 1 && $edtd == 1 }  {
incr crversion
set proj $bproj\_$crversion
vfil 
set edtd 0
set hedited ""
uphead 0
vput
set logtxt "New version $crversion created "
texinfo green "New version $crversion created"
update idletasks
logit
chkbut
after 2000
# reset program buttons to match absent results files
} }


##################### vusum   show summary of this run
proc vusum { sfil } {
# files with names proj.IDS ID is program two letter ID
global proj lid vufil sheader sheader1 atend
debug "vusum "
set atend 1
set vufil $sfil
set sheader $sheader1
viewfile
set sheader ""
}   
##################### vusout   show global summary/output of this project
proc vusout { id typ} {
# files with names proj.IDS ID is program two letter ID, typ S summary O output
global proj lid vufil sheader sheader0 sheader1 atend
debug "vusout"
set sfil "$proj.$id\M"
if { $typ == "O" } { set sfil "$proj.$id\O" }
set atend 1
set vufil $sfil
set sheader ""
if { $typ == "M" } { set sheader "$sheader0\n$sheader1" }
viewfile
set sheader ""
}   




#####################################################
#####################################################
#
#                MAIN
#
######################################################
#####################################################
#
# WINDOWS installation using WIX
#
#https://www.guidgenerator.com/online-guid-generator.aspx
#
#
# master setting for calling debug output - change this and add global in 
# debug printout message
#
#     candle cr2020.wxs
#	use heat to obtain crbits.wxs 
#
#  D:\CRYSFIRE\msi>heat dir ../crexe -dr INSTALLDIR -cg cradds -gg -g1 -var
# var.Srcdir -out  crbits.wxs
#
#  and compile this after edit mods introducing INSTALLDIR to replace directory
#  Srcdir GUID
#    compile with   candle crbits.wxs  -dSrcdir=D:\crysfire2020\crexe    
#
#  then 
#       build with  
#      light cr2020.wixobj crbits.wixobj -ext \
#     "c:\program files\Wix Toolset v3.7\bin\WixUIExtension.dll" \
#       -out cr2020.msi 
#
#   For macintosh the executable files for the programs are in a zip file
#   the gfortran library may have to be added as an environmental variable
#   set DYLD_FALLBACK_LIBRARY_PATH to the executable directory which should contain
#   the gcc/gfortran libaries.  (Older systems may use DYLD_LIBRARY_PATH)

#
set vversion "1.0.8"
wm title . " Crysfire 2020 $vversion"
#wm state normal
set vdate "April-2024"
set min2t 0.8
set max2t 160
set pform $tcl_platform(platform)
set os $tcl_platform(os)
set browsr explorer
if { $pform =="unix" } {
set homedir $env(HOME) 
if { $os == "Darwin" } { set browsr open }
if { $os == "Linux" } { set browsr xdg-open }
}
if {$pform != "unix" } {
set homedir "$env(HOMEDRIVE)$env(HOMEPATH)"
}
set argv1 [ lindex $argv 0 ]
set argv2 [ lindex $argv 1 ]
set ddt 0
if { $argv1 != "" } { set ddt 1 }
if { $ddt == 1 } { set debunit [ open $homedir/crdebug w ] }
# setup fonts using default stored in $homedir/.CRfont value
ftini
ftupt
set sdir ""
#
### we will change directories so sdir must be an absolute path not relative
##  to cwd
#
# find program dir sdir and bat file dir, pdir, from command line of tcl script
set ppwd [ pwd ]
set sdir [ file dirname $argv0 ]
if { $pform == "windows" } {
# second character will be colon for absolute local path 
set colo [string range $sdir 1 1 ]
if { $colo != ":" } { set sdir $ppwd/$sdir }
}
if { $pform == "unix" } {
set cstrt [ string range $sdir 0 0 ]
# note any leading ~ has been removed by shell
if { $cstrt != "/" } { set sdir $ppwd/$sdir }
if { $os == "Darwin" } {
# for Macintosh - fortran runtime libraries
set env(DYLD_FALLBACK_LIBRARY_PATH) $sdir
}
}
#   startup - find previous project(s)  defaults are in ~/.CRroot   etc

ini

frame .mb -relief raised -borderwidth 2 
menubutton .mb.file -text "File" -menu .mb.file.menu
menu .mb.file.menu -tearoff false
.mb.file.menu add command -label "New project" -command { newpron }
.mb.file.menu add command -label "Resume project" -command { resproj }
#.mb.file.menu add command -label "New Work Directory" -command { newcrrt }

.mb.file.menu add command -label "Exit" -command { 
exit }
menubutton .mb.set -text "settings" -menu .mb.set.sets
menu .mb.set.sets -tearoff false
.mb.set.sets add command -label "Change title/wavelength etc." -command { 
if { $bproj == "" } {
message "pleae open or resume project"
}
if { $bproj != "" } {
edtpar } }
.mb.set.sets add command -label "Change peak file" -command {
if { $bproj == "" } {
message "please open or resume project "
}
if { [ file exists $fproj ] } { edtpro }
}
.mb.set.sets add command -label "Change default parameters cr2020.dfp" \
-command { edidef }
.mb.set.sets add command -label "Change work directory" -command {
newcrrt
}
.mb.set.sets add command -label "Change font scaling" -command ftset
menubutton .mb.vu -text "View" -menu .mb.vu.vu
menu .mb.vu.vu -tearoff false
.mb.vu.vu add command -label "Show log file" -command {
set atend 0
set vufil "$bproj.log"
viewfile
}
.mb.vu.vu add command -label "Show cf2020.dfp parameters" -command {
set atend 1
set vufil "cr2020.dfp"
viewfile
}
.mb.vu.vu add command -label "Show cf2020.dfx extras" -command {
set vufil "cr2020.dfx"
viewfile
}
.mb.vu.vu add command -label "Show list of projects " -command { shopro }
.mb.vu.vu add command -label "Show list of project files " -command { shofil }
menubutton .mb.help -text "Help" -menu .mb.help.menu
menu .mb.help.menu -tearoff false
.mb.help.menu add command -label "About" -command {
message "CRYSFIRE-2020\n \
This is a reimplementation of Robin Shirley's MS-DOS CRYSFIRE indexing suite of programs\ \
\n using Tcl/Tk and gfortran to produce a multiplatform suite. \
Not all sources in the last version of CRYSFIRE (2004) have been found.\
\n These 1400 lines of tcl replace  43k lines of Pascal and 15k lines of MS-DOS BAT files
\n Version $vversion $vdate Ron Ghosh, Epsom."
}
.mb.help.menu add command -label "The Crysfire Suite" -command {
set savwd [ pwd ]
cd $sdir
set bbb [ exec $browsr "Crysfire.html" & ]
cd $savwd
}
pack .mb.file .mb.set .mb.vu  -side left
pack .mb.help -side right
pack .mb -side top -fill x
frame .main -bg white
####### comment line
label  .main.tt -text " Please use 'File' to open new or resume existing project " \
-width 100 -fg "red"
label  .main.cur -text $header
pack .main.cur -side bottom -fill x 
pack .main.tt -side bottom -fill x
pack .main .mb 
# now treat set of programs list of buttons and file extensions lbutt lext
set lext ""
set lbutt ""
setupallp 
disbut
if { $bproj != "" } {
 chknew 
 chkbut
 }
